@@ -90,10 +90,6 @@ def upack(q):
     h = np.asarray(h)
     return h[:,0], h[:,1], h[:,2],
 
-# class UnitgradeTextRunner(unittest.TextTestRunner):
-#     def __init__(self, *args, **kwargs):
-#         super().__init__(*args, **kwargs)
-
 class SequentialTestLoader(unittest.TestLoader):
     def getTestCaseNames(self, testCaseClass):
         test_names = super().getTestCaseNames(testCaseClass)
@@ -121,9 +117,10 @@ def evaluate_report(report, question=None, qitem=None, passall=False, verbose=Fa
     dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
     print(b + " v" + __version__ + ", started: " + dt_string+ "\n")
     # print("Started: " + dt_string)
+    report._check_remote_versions() # Check (if report.url is present) that remote files exist and are in sync.
     s = report.title
     if hasattr(report, "version") and report.version is not None:
-        s += " version " + report.version
+        s += f" version {report.version}"
     print(s, "(use --help for options)" if show_help_flag else "")
     # print(f"Loaded answers from: ", report.computed_answers_file, "\n")
     table_data = []
@@ -175,7 +172,9 @@ def evaluate_report(report, question=None, qitem=None, passall=False, verbose=Fa
                 detail['status'] = 'error'
             else:
                 raise Exception("Status not known.")
-            detail = {**detail, **msg}#['message'] = msg
+
+            nice_title = s.title
+            detail = {**detail, **msg, 'nice_title': nice_title}#['message'] = msg
             details[key] = detail
 
         # q_[s._testMethodName] = ("pass", None)
