@@ -8,11 +8,14 @@ import colorama
 import unittest
 import time
 import textwrap
+import urllib.parse
+import requests
+import ast
+import numpy
 
 from unitgrade.runners import UTextResult
 from unitgrade.utils import gprint, Capturing2, Capturing
 colorama.init(autoreset=True)  # auto resets your settings after every output
-import numpy
 
 numpy.seterr(all='raise')
 
@@ -24,13 +27,17 @@ def setup_dir_by_class(C, base_dir):
 class Report:
     title = "report title"
     abbreviate_questions = False # Should the test items start with 'Question ...' or just be q1).
+    version = None # A version number of the report (1.0). Used to compare version numbers with online resources.
+    url = None  # Remote location of this problem.
 
-    version = None
     questions = []
     pack_imports = []
     individual_imports = []
+
+    _remote_check_cooldown_seconds = 1 # Seconds between remote check of report.
     nL = 120  # Maximum line width
     _config = None  # Private variable. Used when collecting results from student computers. Should only be read/written by teacher and never used for regular evaluation.
+    _setup_mode = False # True if test is being run in setup-mode, i.e. will not fail because of bad configurations, etc.
 
     @classmethod
     def reset(cls):
@@ -45,6 +52,10 @@ class Report:
     def _file(self):
         return inspect.getfile(type(self))
 
+    def _is_run_in_grade_mode(self):
+        """ True if this report is being run as part of a grade run. """
+        return self._file().endswith("_grade.py") # Not sure I love this convention.
+
     def _import_base_relative(self):
         if hasattr(self.pack_imports[0], '__path__'):
             root_dir = self.pack_imports[0].__path__[0]
@@ -72,13 +83,13 @@ class Report:
         # Run all tests using standard unittest (nothing fancy).
         loader = unittest.TestLoader()
         for q, _ in self.questions:
-            start = time.time()  # A good proxy for setup time is to
+            start = time.time()  #
             suite = loader.loadTestsFromTestCase(q)
             unittest.TextTestRunner(verbosity=verbosity).run(suite)
             total = time.time() - start
             q.time = total
 
-    def _setup_answers(self, with_coverage=False):
+    def _setup_answers(self, with_coverage=False, verbose=True):
         if with_coverage:
             for q, _ in self.questions:
                 q._with_coverage = True
@@ -87,7 +98,7 @@ class Report:
             q._setup_answers_mode = True
 
         from unitgrade import evaluate_report_student
-        evaluate_report_student(self, unmute=True)
+        evaluate_report_student(self, unmute=verbose, noprogress=not verbose)
 
         # self.main()  # Run all tests in class just to get that out of the way...
         report_cache = {}
@@ -110,6 +121,109 @@ class Report:
             q._cache = payloads[q.__qualname__]
         self._config = payloads['config']
 
+    def _check_remote_versions(self):
+        if self.url is None:
+            return
+        url = self.url
+        if not url.endswith("/"):
+            url += "/"
+        snapshot_file = os.path.dirname(self._file()) + "/unitgrade_data/.snapshot"
+        # print("Sanity checking time using snapshot", snapshot_file)
+        # print("and using self-identified file", self._file())
+
+        if os.path.isfile(snapshot_file):
+            with open(snapshot_file, 'r') as f:
+                t = f.read()
+                if (time.time() - float(t)) < self._remote_check_cooldown_seconds:
+                    return
+        # print("Is this file run in local mode?", self._is_run_in_grade_mode())
+
+        if self.url.startswith("https://gitlab"):
+            # Try to turn url into a 'raw' format.
+            # "https://gitlab.compute.dtu.dk/tuhe/unitgrade_private/-/raw/master/examples/autolab_example_py_upload/instructor/cs102_autolab/report2_test.py?inline=false"
+            # url = self.url
+            url = url.replace("-/tree", "-/raw")
+            # print(url)
+            # "https://gitlab.compute.dtu.dk/tuhe/unitgrade_private/-/tree/master/examples/autolab_example_py_upload/instructor/cs102_autolab"
+            # "https://gitlab.compute.dtu.dk/tuhe/unitgrade_private/-/raw/master/examples/autolab_example_py_upload/instructor/report2_test.py?inline=false"
+            # "https://gitlab.compute.dtu.dk/tuhe/unitgrade_private/-/raw/master/examples/autolab_example_py_upload/instructor/cs102_autolab/report2_test.py?inline=false"
+            raw_url = urllib.parse.urljoin(url, os.path.basename(self._file()) + "?inline=false")
+            # print("Is this file run in local mode?", self._is_run_in_grade_mode())
+            if self._is_run_in_grade_mode():
+                remote_source = requests.get(raw_url).text
+                with open(self._file(), 'r') as f:
+                    local_source = f.read()
+                if local_source != remote_source:
+                    print("\nThe local version of this report is not identical to the remote version which can be found at")
+                    print(self.url)
+                    print("The most likely reason for this is that the remote version was updated by the teacher due to some issue.")
+                    print("You should check if there was an announcement and update the test to the most recent version; most likely")
+                    print("This can be done by running the command")
+                    print("> git pull")
+                    print("You can find the most recent code here:")
+                    print(self.url)
+                    raise Exception(f"Version of grade script does not match the remote version. Please update using git pull")
+                #
+                # # node = ast.parse(text)
+                # # classes = [n for n in node.body if isinstance(n, ast.ClassDef) if n.name == self.__class__.__name__][0]
+                #
+                # # for b in classes.body:
+                # #     print(b.)
+                #     # if b.targets[0].id == "version":
+                #         # print(b)
+                #         # print(b.value)
+                #         version_remote = b.value.value
+                #         break
+                # if version_remote != self.version:
+            else:
+                text = requests.get(raw_url).text
+                node = ast.parse(text)
+                classes = [n for n in node.body if isinstance(n, ast.ClassDef) if n.name == self.__class__.__name__][0]
+                for b in classes.body:
+                    # print(b.)
+                    if b.targets[0].id == "version":
+                        # print(b)
+                        # print(b.value)
+                        version_remote = b.value.value
+                        break
+                if version_remote != self.version:
+                    print("\nThe version of this report", self.version, "does not match the version of the report on git", version_remote)
+                    print("The most likely reason for this is that the remote version was updated by the teacher due to some issue.")
+                    print("You should check if there was an announcement and update the test to the most recent version; most likely")
+                    print("This can be done by running the command")
+                    print("> git pull")
+                    print("You can find the most recent code here:")
+                    print(self.url)
+                    raise Exception(f"Version of test on remote is {version_remote}, which is different than this version of the test {self.version}. Please update your test to the most recent version.")
+
+                for (q,_) in self.questions:
+                    qq = q(skip_remote_check=True)
+                    cfile = qq._cache_file()
+
+                    relpath = os.path.relpath(cfile, os.path.dirname(self._file()))
+                    relpath = relpath.replace("\\", "/")
+                    raw_url = urllib.parse.urljoin(url, relpath + "?inline=false")
+                    # requests.get(raw_url)
+
+                    with open(cfile, 'rb') as f:
+                        b1 = f.read()
+
+                    b2 = requests.get(raw_url).content
+                    if b1 != b2:
+                        print("\nQuestion ", qq.title, "relies on the data file", cfile)
+                        print("However, it appears that this file is missing or in a different version than the most recent found here:")
+                        print(self.url)
+                        print("The most likely reason for this is that the remote version was updated by the teacher due to some issue.")
+                        print("You should check if there was an announcement and update the test to the most recent version; most likely")
+                        print("This can be done by simply running the command")
+                        print("> git pull")
+                        print("to avoid running bad tests against good code, the program will now stop. Please update and good luck!")
+                        raise Exception("The data file for the question", qq.title, "did not match remote source found on git. The test will therefore automatically fail. Please update your test/data files.")
+
+                t = time.time()
+                if os.path.isdir(os.path.dirname(self._file()) + "/unitgrade_data"):
+                    with open(snapshot_file, 'w') as f:
+                        f.write(f"{t}")
 
 def get_hints(ss):
     if ss == None:
@@ -143,7 +257,6 @@ class UTestCase(unittest.TestCase):
     # during setup, and the deploy script must be run many times.
     _setup_answers_mode = False
 
-
     def capture(self):
         if hasattr(self, '_stdout') and self._stdout is not None:
             file = self._stdout
@@ -165,6 +278,7 @@ class UTestCase(unittest.TestCase):
     @classmethod
     def reset(cls):
         print("Warning, I am not sure UTestCase.reset() is needed anymore and it seems very hacky.")
+        raise Exception("reset called in test case. This method is deprecated.")
         cls._outcome = None
         cls._cache = None
         cls._cache2 = None
@@ -251,6 +365,7 @@ class UTestCase(unittest.TestCase):
                                 if "def" in lines2[j] or "class" in lines2[j]:
                                     break
                             from snipper.legacy import gcoms
+
                             fun = lines2[j]
                             comments, _ = gcoms("\n".join(lines2[j:l]))
                             if rel not in cc:
@@ -258,9 +373,6 @@ class UTestCase(unittest.TestCase):
                             cc[rel][fun] = (l, "\n".join(comments))
                             # print("found", rel, fun)
                             self._cache_put((self.cache_id(), 'coverage'), self._covcache)
-        #                 print("ending loop B")
-        #         print("At end of outer loop A")
-        # print("-------------------------------------------- Tear down called")
 
     def shortDescriptionStandard(self):
         sd = super().shortDescription()
@@ -304,10 +416,44 @@ class UTestCase(unittest.TestCase):
         m = self._testMethodName
         return c, m
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args, skip_remote_check=False, **kwargs):
         super().__init__(*args, **kwargs)
         self._load_cache()
         self._assert_cache_index = 0
+        # Perhaps do a sanity check here to see if the cache is up to date? To do that, we must make sure the
+        # cache exists locally.
+        # Find the report class this class is defined within.
+        if skip_remote_check:
+            return
+        import inspect
+
+        # file = inspect.getfile(self.__class__)
+        import importlib, inspect
+        found_reports = []
+        # print("But do I have report", self._report)
+        # print("I think I am module", self.__module__)
+        # print("Importlib says", importlib.import_module(self.__module__))
+        # This will delegate you to the wrong main clsas when running in grade mode.
+        for name, cls in inspect.getmembers(importlib.import_module(self.__module__), inspect.isclass):
+            # print("checking", cls)
+            if issubclass(cls, Report):
+                for q,_ in cls.questions:
+                    if q == self.__class__:
+                        found_reports.append(cls)
+        if len(found_reports) == 0:
+            pass # This case occurs when the report _grade script is being run.
+            # raise Exception("This question is not a member of a report. Very, very odd.")
+        if len(found_reports) > 1:
+            raise Exception("This question is a member of multiple reports. That should not be the case -- don't get too creative.")
+        if len(found_reports) > 0:
+            report = found_reports[0]
+            report()._check_remote_versions()
+
+        # self._get_report_class()
+
+
+    # def _get_report_class(self):
+    #     pass
 
     def _ensure_cache_exists(self):
         if not hasattr(self.__class__, '_cache') or self.__class__._cache == None:
@@ -378,10 +524,7 @@ class UTestCase(unittest.TestCase):
         assert(np.all(np.isinf(a1) == np.isinf(a2)))  # Check infinite part.
         a1[np.isinf(a1)] = 0
         a2[np.isinf(a2)] = 0
-
         diff = np.abs(a1 - a2)
-
-        # print(a1, a2, diff)
         return diff
 
 
@@ -455,19 +598,15 @@ class UTestCase(unittest.TestCase):
         cfile = self._cache_file()
         if os.path.exists(cfile):
             try:
-                # print("\ncache file", cfile)
                 with open(cfile, 'rb') as f:
                     data = pickle.load(f)
                 self.__class__._cache = data
             except Exception as e:
-                print("Bad cache", cfile)
+                print("Cache file did not exist:", cfile)
                 print(e)
         else:
             print("Warning! data file not found", cfile)
 
-    # def _feedFailuresToResult(self, result, errors):
-    #     print("asdfdf")
-
     def _feedErrorsToResult(self, result, errors):
         """ Use this to show hints on test failure. """
         if not isinstance(result, UTextResult):
@@ -479,7 +618,8 @@ class UTestCase(unittest.TestCase):
                 if self._cache_contains(key):
                     CC = self._cache_get(key)
                     cl, m = self.cache_id()
-                    gprint(f"> An error occured while solving: {cl}.{m}. The files/methods you need to edit are:")  # For the test {id} in {file} you should edit:")
+                    # Insert newline to get better formatting.
+                    gprint(f"\n> An error occured during the test: {cl}.{m}. The following files/methods has code in them you are supposed to edit and may therefore be the cause of the problem:")
                     for file in CC:
                         rec = CC[file]
                         gprint(f">   * {file}")
@@ -519,6 +659,20 @@ class UTestCase(unittest.TestCase):
     def startTestRun(self):
         super().startTestRun()
 
+class Required:
+    pass
+
+class ParticipationTest(UTestCase,Required):
+    max_group_size = None
+    students_in_group = None
+    workload_assignment = {'Question 1': [1, 0, 0]}
+
+    def test_students(self):
+        pass
+
+    def test_workload(self):
+        pass
+
 # 817, 705
 class NotebookTestCase(UTestCase):
     notebook = None
