@@ -13,7 +13,7 @@ _CacheInfo = namedtuple("CacheInfo", ["hits", "misses", "maxsize", "currsize"])
 
 
 def gprint(s):
-    print(f"{Fore.GREEN}{s}")
+    print(f"{Fore.LIGHTGREEN_EX}{s}")
 
 
 myround = lambda x: np.round(x)  # required for obfuscation.
@@ -97,11 +97,11 @@ def rm_progress_bar(txt):
 
 
 class ActiveProgress():
-    def __init__(self, t, start=True, title="my progress bar", show_progress_bar=True, file=None):
+    def __init__(self, t, start=True, title="my progress bar", show_progress_bar=True, file=None, mute_stdout=False):
         if file == None:
             file = sys.stdout
         self.file = file
-        # self.t = t
+        self.mute_stdout = mute_stdout
         self._running = False
         self.title = title
         self.dt = 0.025
@@ -113,6 +113,12 @@ class ActiveProgress():
             self.start()
 
     def start(self):
+        if self.mute_stdout:
+            import io
+            # from unitgrade.utils import Logger
+            self._stdout = sys.stdout
+            sys.stdout = Logger(io.StringIO(), write_to_stdout=False)
+
         self._running = True
         if self.show_progress_bar:
             self.thread = threading.Thread(target=self.run)
@@ -133,6 +139,14 @@ class ActiveProgress():
             self.pbar = None
 
         self.file.flush()
+
+        if self.mute_stdout:
+            import io
+            # from unitgrade.utils import Logger
+            sys.stdout = self._stdout #= sys.stdout
+
+            # sys.stdout = Logger(io.StringIO(), write_to_stdout=False)
+
         return time.time() - self.time_started
 
     def run(self):
